﻿using System;
using System.Runtime.InteropServices;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace RP2005_CSharp
{
  public partial class Form1 : Form
  {
      //Get information concerning the devices currently connected. This function returns the number of devices
      //connected, and each device’s serial number and product description.
      [DllImport("rp2005.dll")]
      public static extern uint RP_ListDIO(ref uint numberOfBoards, StringBuilder serialNumber, StringBuilder description);

      [DllImport("rp2005.dll")]
      public static extern uint RP_OpenDIO(string SN, ref uint hDIO);

      [DllImport("rp2005.dll")]
      public static extern uint RP_CloseDIO(uint hDIO);

      [DllImport("rp2005.dll")]
      public static extern uint RP_GetVer(uint hDIO, StringBuilder SwVer, StringBuilder ErrMsg);

      [DllImport("rp2005.dll")]
      public static extern uint RP_ReadPort(uint hDIO, byte ucPort, ref byte PVal, StringBuilder ErrMsg);

      [DllImport("rp2005.dll")]
      public static extern uint RP_WritePort(uint hDIO, byte ucPort, byte PVal, StringBuilder ErrMsg);

      [DllImport("rp2005.dll")]
      public static extern uint RP_ReadAll(uint hDIO, ref uint ulLP, ref uint ulHP, StringBuilder ErrMsg);

      [DllImport("rp2005.dll")]
      public static extern uint RP_WriteAll(uint hDIO, uint ulLP, uint ulHP, StringBuilder ErrMsg);
      uint uiDIO = 0;

    public Form1()
    {
      InitializeComponent();
    }

    private void btnGetVersion_Click(object sender, EventArgs e)
    {
      uint result = 0;
      uint hDIO = 0;
      StringBuilder sbSWV = new StringBuilder(20);
      StringBuilder sbErrMsg = new StringBuilder(512);

      sbErrMsg.Length = 0;
      if (cmbSerialNumbers.Text.Length > 0)
      {
        result = RP_OpenDIO(cmbSerialNumbers.Text, ref hDIO); ;
        if (result == 0)
        {
//          uiDIO = hDIO;
          result = RP_GetVer(hDIO, sbSWV, sbErrMsg);
          sbSWV.Insert(0, "Version : ");
          lblSWVer.Text = sbSWV.ToString();
          if (result != 0) sbErrMsg.Append(" Error Getting Version - " + result.ToString());
        }
        else sbErrMsg.Append("Error Opening Device - " + result.ToString());
        if (hDIO > 0) 
        {
          result = RP_CloseDIO(hDIO);
          if (result != 0) sbErrMsg.Append("Error Closing Device - " + result.ToString());
        }
      }
      toolStripStatusLabel1.Text = sbErrMsg.ToString();
    }

    private void btnGetDevices_Click(object sender, EventArgs e)
    {
      uint uiNumBrds = 0;
      StringBuilder sbSN = new StringBuilder(512);
      StringBuilder sbDesc = new StringBuilder(512);

      uint result = RP_ListDIO(ref uiNumBrds, sbSN, sbDesc);
      if ((result == 0) && (uiNumBrds > 0))
      {
        cmbSerialNumbers.Items.Clear();
        cmbSerialNumbers.Items.AddRange(sbSN.ToString().Split(','));
        cmbSerialNumbers.Text = cmbSerialNumbers.Items[0].ToString();
      }
    }

    private void btnReadAll_Click(object sender, EventArgs e)
    {
      uint result = 0;
      uint hDIO = 0;
      uint uiLP = 0xffffffff, uiHP = 0xffffffff;
      uint uiInPort = 0, uiOutPort = 0;
      int i;
      StringBuilder sbErrMsg = new StringBuilder(512);

      sbErrMsg.Length = 0;
      if (cmbSerialNumbers.Text.Length > 0)
      {
        result = RP_OpenDIO(cmbSerialNumbers.Text, ref hDIO); ;
        if (result == 0)
        {
          result = RP_ReadAll(hDIO, ref uiLP, ref uiHP, sbErrMsg);
          if (result == 0)
          {
            for (i = 3; i >= 0; i--)
            {
              uiInPort = uiLP & 0xff;
              uiOutPort = uiHP & 0xff;
              switch (i)
              {
                case 0:
                  tbxPort0.Text = uiInPort.ToString();
                  tbxPort10.Text = uiOutPort.ToString();
                  break;
                case 1:
                  tbxPort1.Text = uiInPort.ToString();
                  tbxPort11.Text = uiOutPort.ToString();
                  break;
                case 2:
                  tbxPort2.Text = uiInPort.ToString();
                  tbxPort12.Text = uiOutPort.ToString();
                  break;
                case 3:
                  tbxPort3.Text = uiInPort.ToString();
                  tbxPort13.Text = uiOutPort.ToString();
                  break;
              }
              uiLP = uiLP >> 8;
              uiHP = uiHP >> 8;
            }
          }
          else
          {
            toolStripStatusLabel1.Text = sbErrMsg.ToString();
            result = RP_CloseDIO(hDIO);
            return;
          }
          if (result != 0) sbErrMsg.Append(" Error Getting Version - " + result.ToString());
        }
        else sbErrMsg.Append("Error Opening Device - " + result.ToString());
        if (hDIO > 0)
        {
          result = RP_CloseDIO(hDIO);
          if (result != 0) sbErrMsg.Append("Error Closing Device - " + result.ToString());
        }
      }
      toolStripStatusLabel1.Text = sbErrMsg.ToString();
    }

    private void btnWriteAll_Click(object sender, EventArgs e)
    {
      uint result = 0;
      uint hDIO = 0;
      uint uiLP = 0xFFFFFFFF, uiHP = 0xFFFFFFFF;
      int i;
      StringBuilder sbErrMsg = new StringBuilder(512);

      sbErrMsg.Length = 0;
      if (cmbSerialNumbers.Text.Length > 0)
      {
        result = RP_OpenDIO(cmbSerialNumbers.Text, ref hDIO); ;
        if (result == 0)
        {
          for (i = 0; i <= 3; i++)
          {
            uiLP <<= 8;
            uiHP <<= 8;
            switch (i)
            {
              case 0:
                uiLP += Convert.ToUInt32(tbxPort10.Text);
                break;
              case 1:
                uiLP += Convert.ToUInt32(tbxPort11.Text);
                break;
              case 2:
                uiLP += Convert.ToUInt32(tbxPort12.Text);
                break;
              case 3:
                uiLP += Convert.ToUInt32(tbxPort13.Text);
                break;
            }
          }
          result = RP_WriteAll(hDIO, uiLP, uiHP, sbErrMsg);
          if (result != 0) sbErrMsg.Append(" Error Writing Ports - " + result.ToString());
        }
        else sbErrMsg.Append("Error Opening Device - " + result.ToString());
        if (hDIO > 0)
        {
          result = RP_CloseDIO(hDIO);
          if (result != 0) sbErrMsg.Append("Error Closing Device - " + result.ToString());
        }
      }
      toolStripStatusLabel1.Text = sbErrMsg.ToString();
    }
  }
}
